
#include <X11/Xlib.h>
#include <X11/extensions/XTest.h>
#include <GL/glew.h>
#include <GL/glut.h>
#include <GL/freeglut_ext.h>
#include <GL/glx.h>
#include <GL/glxext.h>
#include <stdio.h>
#include <unistd.h>


void e(const char* description)
{
	if (description && description[0]) {
		fprintf(stderr, "e:%s\n",  description);
	}
	GLenum err;
	while((err = glGetError()) != GL_NO_ERROR)
	{
	    fprintf(stderr, "opengl error: %x\n", (unsigned int)err);
	}
}

int main(int argc, char** argv)
{
	Display* dpy = XOpenDisplay(NULL);

    int xtestEventBase;
    int xtestErrorBase;
    int xtestMajorVersion;
    int xtestMinorVersion;

    Bool have_XTestExtension =
	XTestQueryExtension(
	    dpy,
	    &xtestEventBase,
	    &xtestErrorBase,
	    &xtestMajorVersion,
	    &xtestMinorVersion
	);

    if (have_XTestExtension) {
	//become immune to server grabs
	XTestGrabControl(dpy, /* impervious = */ True);
	printf("became impervious to server grabs\n");
    }

	unsigned int width = 700;
	unsigned int height = 500;

	int screenNum = DefaultScreen(dpy);

	Window w = XCreateSimpleWindow(dpy, RootWindow(dpy, screenNum), 0, 0, width, height, 2, 0, 0xffffff);
	XMapRaised(dpy, w);

	// Choose a visual
	XVisualInfo *pvi;

	int attrib[]=
	    { GLX_RGBA,
	      GLX_RED_SIZE, 8,
	      GLX_GREEN_SIZE, 8,
	      GLX_BLUE_SIZE, 8,
	      GLX_ALPHA_SIZE, 0,
	      None };	
	    glewInit();


	pvi=glXChooseVisual(dpy, screenNum, attrib);
	if(pvi==NULL) {
	    fprintf(stderr, "failure to choose GLX-Visual. quitting.\n");
	    fflush(stderr);
	    exit(1);
	}

	GLXContext glxContext = 
		glXCreateContext(dpy, pvi, NULL, /* direct = */ True);
	if(glxContext==NULL) {
	    fprintf(stderr, "failure to create GLXContext. quitting.\n");
	    fflush(stderr);
	    exit(1);
	}
	glXMakeCurrent(dpy, w, glxContext);
	//glewInit();

	glDrawBuffer(GL_FRONT);

	sleep(1);


	glLineWidth(2.5);
	glColor3f(1.0, 0.0, 0.0);

	while(1) {
		XClearWindow(dpy, w);
		for (unsigned i = 0; i < width; i++) {
			glBegin(GL_LINES);
			    glVertex2f(2.0 * (double)i / (double)width - 1.0, -1.0);
			    glVertex2f(2.0 * (double)(width-i)/(double)(width) - 1.0, 1.0);
			glEnd();
			e("");
			glFlush();
			XFlush(dpy);

			e("");

			usleep( 10000000 / width );
		}
	}

	XCloseDisplay(dpy);

	return 0;
}
